
import numpy
from functools import reduce
from operator import mul

class real:
    d = 1

class vector:
    def __init__(self, d):
        self.d = d

class Results:
    def __init__(self, cfgf="config.py", dataf="vars.dat"):
        self.cfgf = cfgf
        self.dataf = dataf
        v = dict(real=real, vector=vector)
        with open(cfgf) as f:
            code = compile(f.read(), cfgf, 'exec')
            exec(code, v, v)
        self.c = v['config']
        if self.c['DOUBLE']:
            self.dt = numpy.float64
        else:
            self.dt = numpy.float32
        dt = self.c['MAXT']/self.c['NT']
        if self.c['T0']:
            self.nt = self.c['NT'] + 1
            self.t = [dt*n for n in range(self.nt)]
        else:
            self.nt = self.c['NT']
            self.t = [dt*(n+1) for n in range(self.nt)]
        self.nc = reduce(mul, self.c['NXYZ'])
        i = 0
        self.vardesc = {}
        for n,t in self.c['VARS']:
            t.i = i
            self.vardesc[n] = t
            i += t.d

    def allvars(self):
        d = numpy.fromfile(self.dataf, self.dt)
        nv = d.shape[0] / (self.nt * self.nc)
        d.shape = (self.nt, nv)+tuple(self.c['NXYZ'])
        return d

    def var(self, name, c=None, t=None, getextent=False):
        """Return named variable.

        c is the direction component for vector variables.  If c is not
          given then all components are returned.
        t is the timestep index.  If t is not given all times are returned.
        """
        off = 0
        if t is not None:
            if t < 0: t += self.nt
            assert t >= 0 and t < self.nt
            off = t*self.c['VARSIZE']
            count = 1
            shape = []
            ex = []
        else:
            count = self.nt
            shape = [count]
            ex = [(self.t[0], self.c['MAXT'])]
        vt = self.vardesc[name]
        if c is not None:
            assert isinstance(vt, vector)
            assert c >= 0 and c < vt.d
            off += c
        elif isinstance(vt, vector):
            count *= vt.d
            shape.append(vt.d)
            ex.append((0,vt.d))
        shape = tuple(shape+self.c['NXYZ'])
        ex.extend([(0,m) for m in self.c['MAXXYZ']])
        off *= self.nc * ( 8 if self.c['DOUBLE'] else 4 )
        count *= self.nc
        f = open(self.dataf, 'rb')
        f.seek(off)
        d = numpy.fromfile(f, dtype=numpy.float64, count=count)
        d.shape = shape
        if getextent: return d,ex
        return d

    def plot(self, name, c=None, t=-1):
        from matplotlib import pyplot
        if t is not None and t < 0: t += self.nt
        d, ex = self.var(name, c, t, getextent=True)
        if len(d.shape) == 1:
            ex = ex[0]
            x = ex[0]+numpy.arange(d.shape[0])*(ex[1]-ex[0])/d.shape[0]
            pyplot.plot(x, d)
        elif len(d.shape) == 2:
            ex = (ex[0][0], ex[0][1], ex[1][0], ex[1][1])
            pyplot.imshow(d, extent=ex)
            pyplot.colorbar()
        else:
            assert False
        if t is not None:
            pyplot.title("{} t={}".format(name, self.t[t]))
        else:
            pyplot.title(name)
        pyplot.show()

